@@ -11,10 +11,10 @@ from dataclasses import dataclass
 from pandas import DataFrame
 import numpy as np
 from scipy.stats import kstest, normaltest, shapiro
-from pyjr.utils._tools.math import _min, _max, _mean, _var, _std, _sum, _med, _mod, _skew, _kurt, _perc
-from pyjr.utils._tools.math import _percs, _range
-from pyjr.utils._tools.clean import _nan, _prep, _mtype
-from pyjr.utils._tools.general import _unique_values
+from pyjr.utils.tools.math import _min, _max, _mean, _var, _std, _sum, _med, _mod, _skew, _kurt, _perc, _percs, _range
+from pyjr.utils.tools.math import _gini
+from pyjr.utils.tools.clean import _nan, _prep, _mtype
+from pyjr.utils.tools.general import _unique_values
 
 
 @dataclass
@@ -45,7 +45,7 @@ class Data:
     """
 
     __slots__ = ("name", "data", "len", "unique", "dtype", "mean", "median", "mode", "var", "std", "lower", "higher",
-                 "min", "max", "sum", "skew", "kurt", "per", "rang", "distribution", "na")
+                 "min", "max", "sum", "skew", "kurt", "per", "rang", "distribution", "na", "gini")
 
     def __init__(self, data = None, name: str = None, na_handling: str = 'none',
                  dtype: str = 'float', cap_zero: bool = True, std_value: int = 3, median_value: float = 0.023,
@@ -76,10 +76,11 @@ class Data:
             self.kurt = _kurt(d=self.data)
             self.per = _perc(d=self.data, q=0.75)
             self.rang = _range(d=self.data)
+            self.gini = _gini(d=self.data)
         else:
             self.mean, self.median, self.mode, self.var, self.std = None, None, None, None, None
             self.lower, self.higher, self.min, self.max, self.sum = None, None, None, None, None
-            self.skew, self.kurt, self.per, self.rang = None, None, None, None
+            self.skew, self.kurt, self.per, self.rang, self.gini = None, None, None, None, None
 
         self.na = None
         if self.data.__len__() != data.__len__():
@@ -119,12 +120,18 @@ class Data:
         """Returns a tuple"""
         return tuple(self.data)
 
-    def array(self, axis: int = 0) -> np.ndarray:
+    def array(self, axis: int = 0, ts: bool = False) -> np.ndarray:
         """Returns an np.ndarray"""
-        if axis == 0:
-            return np.array(self.data)
+        if ts:
+            arr = np.ones((self.data.__len__(), 2))
+            arr[:, 0] = self.data
+            arr[:, 1] = list(range(self.data.__len__()))
+            return arr.reshape((2, self.data.__len__()))
         else:
-            return np.array(self.data).reshape(self.len, 1)
+            if axis == 0:
+                return np.array(self.data)
+            else:
+                return np.array(self.data).reshape(self.len, 1)
 
     def dataframe(self, index: list = None, name: str = None) -> DataFrame:
         """Returns a pd.DataFrame"""
